/** @file Membrane22Rtsc.h
 * @brief Contains the interface of the Membrane22Rtsc class
 *
 * @attention All rights reserved; do not distribute without permission.
 * @author (C) Copyright 2011-2012 Paolo Greppi Universita di Genova; (C) Copyright 2012-2024 Paolo Greppi simevo s.r.l. 
 */

#ifndef MEMBRANE22RTSC_H
#define MEMBRANE22RTSC_H

/* SYSTEM INCLUDES */
//

/* PROJECT INCLUDES */
#include <libpf/units/MultiExchanger.h>

/* LOCAL INCLUDES */

/* FORWARD REFERENCES */
class MultiReactionTransfer;

/// Two-feeds, two-products, RTSC, concentrated parameter membrane model
class Membrane22Rtsc : public MultiExchanger {
public:
  // LIFECYCLE  
  Membrane22Rtsc(Libpf::Persistency::Defaults defaults, uint32_t id=0, Persistency *persistency=nullptr, Persistent *parent=nullptr, Persistent *root=nullptr);
  
  // CUSTOM FLOWSHEET VARIABLES
  // inputs
  std::vector<Quantity> permeance; ///< permeance for each component
  Quantity deltaPretentate; ///< retentate compartment pressure drop
  Quantity deltaPpermeate; ///< permeate compartment pressure drop  

  // computed
  Quantity Tavg; ///< Mol-weighted average inlet temperature
  Quantity stageCut; ///< fraction of feed to permeate
  Quantity delta; ///< pressure ratio
  Quantity N; ///< feeds total mole flow
  std::vector<Quantity> permeationFactor; ///< permeation factor for each component (Ri)
  std::vector<Quantity> lambda; ///< average feed molar fractions

  // OPERATIONS
  // mandatory
  const std::string &type(void) const { return type_; }
  // non-mandatory
  void go(SolutionMode solutionMode, int level);
private: 
  static std::string type_;
  void pre_calculate(void);
  void post_calculate(void);
  MultiReactionTransfer *mrt_;  
};  // class Membrane22Rtsc

#endif // MEMBRANE22RTSC_H
