/** @file MembraneMultiRtsc.h
 * @brief Contains the interface of the MembraneMulti22RtscCoflow and MembraneMulti22RtscCounterflow classes
 *
 * @attention All rights reserved; do not distribute without permission.
 * @author (C) Copyright 2011-2012 Paolo Greppi Universita di Genova; (C) Copyright 2012-2024 Paolo Greppi simevo s.r.l.
 */

#ifndef MEMBRANEMULTIRTSC_H
#define MEMBRANEMULTIRTSC_H

/* SYSTEM INCLUDES */
//

/* PROJECT INCLUDES */
#include <libpf/flowsheet/FlowSheet.h>

/* LOCAL INCLUDES */

/* FORWARD REFERENCES */
//

// /////////////////////////////////////////////////////////////////////////////
// ////////////////////////// MembraneMulti22RtscCoflow ////////////////////////
// /////////////////////////////////////////////////////////////////////////////

/// Two-feeds, two-products, RTSC, co-flow, multi-stage membrane model
/// Options:
///  - stages total number of stages, default value is 10
class MembraneMulti22RtscCoflow : virtual public Model, public FlowSheet {
public:
  // LIFECYCLE  
  MembraneMulti22RtscCoflow(Libpf::Persistency::Defaults defaults, uint32_t id=0, Persistency *persistency=nullptr, Persistent *parent=nullptr, Persistent *root=nullptr);
  
  // CUSTOM FLOWSHEET VARIABLES
  // inputs
  Quantity A; ///< membrane area
  Quantity deltaPretentate; ///< retentate compartment pressure drop
  Quantity deltaPpermeate; ///< permeate compartment pressure drop
  std::vector<Quantity> permeance; ///< permeance for each component

  // computed
  Quantity stageCut; ///< fraction of feed to permeate
  std::vector<Quantity> outSplit;  ///< component split fractions to permeate

  Integer stages; ///< total number of stages

  // OPERATIONS
  // mandatory
  void setup(void);
  const std::string &type(void) const { return type_; }
  void makeUserEquations(std::list<Assignment *>::iterator &p) { }
  void pre(Calculatable::SolutionMode solutionMode, int level);
  void post(Calculatable::SolutionMode solutionMode, int level);
  // non-mandatory
  // bool supportsSimultaneous(void) { return false; }
  // int maximumIterations(void) { return 1; }
private:
  static std::string type_;
  std::string stagename(int i);
  const std::string reteout(void);
  const std::string permout(void);
};  // class MembraneMulti22RtscCoflow

// /////////////////////////////////////////////////////////////////////////////
// ////////////////////// MembraneMulti22RtscCounterflow ///////////////////////
// /////////////////////////////////////////////////////////////////////////////

/// Two-feeds, two-products, RTSC, counter-flow, multi-stage membrane model
/// Options:
///  - stages total number of stages, default value is 10
class MembraneMulti22RtscCounterflow : virtual public Model, public FlowSheet {
public:
  // LIFECYCLE  
  MembraneMulti22RtscCounterflow(Libpf::Persistency::Defaults defaults, uint32_t id=0, Persistency *persistency=nullptr, Persistent *parent=nullptr, Persistent *root=nullptr);
  
  // CUSTOM FLOWSHEET VARIABLES
  // inputs
  Quantity A; ///< membrane area
  Quantity deltaPretentate; ///< retentate compartment pressure drop
  Quantity deltaPpermeate; ///< permeate compartment pressure drop
  std::vector<Quantity> permeance; ///< permeance for each component
  std::vector<Quantity> outSplit;  ///< component split fractions to permeate

  // computed
  Quantity stageCut; ///< fraction of feed to permeate

  Integer stages; ///< total number of stages

  // OPERATIONS
  // mandatory
  void setup(void);
  const std::string &type(void) const { return type_; }
  void makeUserEquations(std::list<Assignment *>::iterator &p) { }
  void pre(Calculatable::SolutionMode solutionMode, int level);
  void post(Calculatable::SolutionMode solutionMode, int level);
  // non-mandatory
  // bool supportsSimultaneous(void) { return false; }
  // int maximumIterations(void) { return 1; }
  int sequential(void);
private:
  static std::string type_;
  std::string stagename(int i);
  const std::string reteout(void);
  const std::string permout(void);
};  // class MembraneMulti22RtscCounterflow

#endif // MEMBRANEMULTIRTSC_H
